@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2008 Red Hat, Inc.
+ * (C) Copyright 2008 - 2010 Red Hat, Inc.
  */
 
 #include <string.h>
@@ -33,6 +33,8 @@
 
 #include <nm-setting-connection.h>
 #include <nm-setting-ip4-config.h>
+#include <nm-setting-wired.h>
+#include <nm-setting-wireless.h>
 #include <nm-setting-gsm.h>
 #include <nm-setting-cdma.h>
 #include <nm-setting-pppoe.h>
@@ -78,10 +80,17 @@ typedef struct {
 	GtkWidget *dhcp_client_id_label;
 	GtkEntry *dhcp_client_id;
 
+	/* Routes */
 	GtkButton *routes_button;
 
+	/* IPv4 required */
+	GtkCheckButton *ip4_required;
+
 	GtkWindowGroup *window_group;
 	gboolean window_added;
+
+	/* Cached tree view entry for editing-canceled */
+	char *last_edited;
 } CEPageIP4Private;
 
 #define METHOD_COL_NAME 0
@@ -92,6 +101,7 @@ typedef struct {
 #define IP4_METHOD_MANUAL          2
 #define IP4_METHOD_LINK_LOCAL      3
 #define IP4_METHOD_SHARED          4
+#define IP4_METHOD_DISABLED        5
 
 static void
 ip4_private_init (CEPageIP4 *self, NMConnection *connection)
@@ -172,6 +182,15 @@ ip4_private_init (CEPageIP4 *self, NMConnection *connection)
 		                    -1);
 	}
 
+	/* At the moment, Disabled is only supported for Wired & WiFi */
+	if (   priv->connection_type == NM_TYPE_SETTING_WIRED
+	    || priv->connection_type == NM_TYPE_SETTING_WIRELESS) {
+		gtk_list_store_append (priv->method_store, &iter);
+		gtk_list_store_set (priv->method_store, &iter,
+		                    METHOD_COL_NAME, _("Disabled"),
+		                    METHOD_COL_NUM, IP4_METHOD_DISABLED,
+		                    -1);
+	}
 	gtk_combo_box_set_model (priv->method, GTK_TREE_MODEL (priv->method_store));
 
 	priv->addr_label = glade_xml_get_widget (xml, "ip4_addr_label");
@@ -197,6 +216,14 @@ ip4_private_init (CEPageIP4 *self, NMConnection *connection)
 		gtk_widget_hide (GTK_WIDGET (priv->dhcp_client_id));
 	}
 
+	priv->ip4_required = GTK_CHECK_BUTTON (glade_xml_get_widget (xml, "ip4_required_checkbutton"));
+	/* Hide IP4-require button if it'll never be used for a particular method */
+	if (   priv->connection_type == NM_TYPE_SETTING_VPN
+	    || priv->connection_type == NM_TYPE_SETTING_GSM
+	    || priv->connection_type == NM_TYPE_SETTING_CDMA
+	    || priv->connection_type == NM_TYPE_SETTING_PPPOE)
+		gtk_widget_hide (GTK_WIDGET (priv->ip4_required));
+
 	priv->routes_button = GTK_BUTTON (glade_xml_get_widget (xml, "ip4_routes_button"));
 }
 
@@ -209,6 +236,7 @@ method_changed (GtkComboBox *combo, gpointer user_data)
 	gboolean dns_enabled = FALSE;
 	gboolean dhcp_enabled = FALSE;
 	gboolean routes_enabled = FALSE;
+	gboolean ip4_required_enabled = TRUE;
 	GtkTreeIter iter;
 
 	if (gtk_combo_box_get_active_iter (priv->method, &iter)) {
@@ -228,6 +256,8 @@ method_changed (GtkComboBox *combo, gpointer user_data)
 	case IP4_METHOD_MANUAL:
 		addr_enabled = dns_enabled = routes_enabled = TRUE;
 		break;
+	case IP4_METHOD_DISABLED:
+		addr_enabled = dns_enabled = dhcp_enabled = routes_enabled = ip4_required_enabled = FALSE;
 	default:
 		break;
 	}
@@ -267,6 +297,8 @@ method_changed (GtkComboBox *combo, gpointer user_data)
 	if (!dhcp_enabled)
 		gtk_entry_set_text (priv->dhcp_client_id, "");
 
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->ip4_required), ip4_required_enabled);
+
 	gtk_widget_set_sensitive (GTK_WIDGET (priv->routes_button), routes_enabled);
 
 	ce_page_changed (CE_PAGE (user_data));
@@ -314,6 +346,8 @@ populate_ui (CEPageIP4 *self)
 			method = IP4_METHOD_MANUAL;
 		else if (!strcmp (str_method, NM_SETTING_IP4_CONFIG_METHOD_SHARED))
 			method = IP4_METHOD_SHARED;
+		else if (!strcmp (str_method, NM_SETTING_IP4_CONFIG_METHOD_DISABLED))
+			method = IP4_METHOD_DISABLED;
 	}
 
 	if (method == IP4_METHOD_AUTO && nm_setting_ip4_config_get_ignore_auto_dns (setting))
@@ -391,6 +425,10 @@ populate_ui (CEPageIP4 *self)
 			                    nm_setting_ip4_config_get_dhcp_client_id (setting));
 		}
 	}
+
+	/* IPv4 required */
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->ip4_required),
+	                              !nm_setting_ip4_config_get_may_fail (setting));
 }
 
 static void
@@ -414,16 +452,11 @@ addr_add_clicked (GtkButton *button, gpointer user_data)
 	path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
 	column = gtk_tree_view_get_column (priv->addr_list, COL_ADDRESS);
 
-#if GTK_CHECK_VERSION(2,12,0)
-	cells = gtk_cell_layout_get_cells (GTK_CELL_LAYOUT (column));
-#else
-	cells = gtk_tree_view_column_get_cell_renderers (column);
-#endif
-
 	/* FIXME: using cells->data is pretty fragile but GTK apparently doesn't
 	 * have a way to get a cell renderer from a column based on path or iter
 	 * or whatever.
 	 */
+	cells = gtk_cell_layout_get_cells (GTK_CELL_LAYOUT (column));
 	gtk_tree_view_set_cursor_on_cell (priv->addr_list, path, column, cells->data, TRUE);
 
 	g_list_free (cells);
@@ -474,6 +507,37 @@ list_selection_changed (GtkTreeSelection *selection, gpointer user_data)
 		gtk_widget_set_sensitive (button, FALSE);
 }
 
+static void
+cell_editing_canceled (GtkCellRenderer *renderer, gpointer user_data)
+{
+	CEPageIP4 *self;
+	CEPageIP4Private *priv;
+	GtkTreeModel *model = NULL;
+	GtkTreeSelection *selection;
+	GtkTreeIter iter;
+	guint32 column;
+
+	/* user_data disposed? */
+	if (GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (renderer), "ce-page-not-valid")))
+		return;
+
+	self = CE_PAGE_IP4 (user_data);
+	priv = CE_PAGE_IP4_GET_PRIVATE (self);
+
+	if (priv->last_edited) {
+		selection = gtk_tree_view_get_selection (priv->addr_list);
+		if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+			column = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (renderer), "column"));
+			gtk_list_store_set (GTK_LIST_STORE (model), &iter, column, priv->last_edited, -1);
+		}
+
+		g_free (priv->last_edited);
+		priv->last_edited = NULL;
+
+		ce_page_changed (CE_PAGE (self));
+	}
+}
+
 static void
 cell_edited (GtkCellRendererText *cell,
              const gchar *path_string,
@@ -488,10 +552,34 @@ cell_edited (GtkCellRendererText *cell,
 	guint32 column;
 	GtkTreeViewColumn *next_col;
 
+	g_free (priv->last_edited);
+	priv->last_edited = NULL;
+
 	column = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (cell), "column"));
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path);
 	gtk_list_store_set (store, &iter, column, new_text, -1);
 
+	/* Try to autodetect the prefix from the given address if we can */
+	if (column == COL_ADDRESS && new_text && strlen (new_text)) {
+		char *prefix = NULL;
+		const char *guess_prefix = NULL;
+
+		gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, path_string);
+		gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, COL_PREFIX, &prefix, -1);
+		if (!prefix || !strlen (prefix)) {
+			if (!strncmp ("10.", new_text, 3))
+				guess_prefix = "8";
+			else if (!strncmp ("172.16.", new_text, 7))
+				guess_prefix = "16";
+			else if (!strncmp ("192.168.", new_text, 8))
+				guess_prefix = "24";
+
+			if (guess_prefix)
+				gtk_list_store_set (store, &iter, COL_PREFIX, guess_prefix, -1);
+		}
+		g_free (prefix);
+	}
+
 	/* Move focus to the next column */
 	column = (column >= COL_LAST) ? 0 : column + 1;
 	next_col = gtk_tree_view_get_column (priv->addr_list, column);
@@ -507,11 +595,13 @@ ip_address_filter_cb (GtkEntry *   entry,
                       const gchar *text,
                       gint         length,
                       gint *       position,
-                      gpointer     data)
+                      gpointer     user_data)
 {
+	CEPageIP4 *self = CE_PAGE_IP4 (user_data);
+	CEPageIP4Private *priv = CE_PAGE_IP4_GET_PRIVATE (self);
 	GtkEditable *editable = GTK_EDITABLE (entry);
 	int i, count = 0;
-	gchar *result = g_new (gchar, length);
+	gchar *result = g_new0 (gchar, length);
 
 	for (i = 0; i < length; i++) {
 		if ((text[i] >= '0' && text[i] <= '9') || (text[i] == '.'))
@@ -521,11 +611,13 @@ ip_address_filter_cb (GtkEntry *   entry,
 	if (count > 0) {
 		g_signal_handlers_block_by_func (G_OBJECT (editable),
 		                                 G_CALLBACK (ip_address_filter_cb),
-		                                 data);
+		                                 user_data);
 		gtk_editable_insert_text (editable, result, count, position);
+		g_free (priv->last_edited);
+		priv->last_edited = g_strdup (gtk_editable_get_chars (editable, 0, -1));
 		g_signal_handlers_unblock_by_func (G_OBJECT (editable),
 		                                   G_CALLBACK (ip_address_filter_cb),
-		                                   data);
+		                                   user_data);
 	}
 
 	g_signal_stop_emission_by_name (G_OBJECT (editable), "insert-text");
@@ -536,17 +628,23 @@ static void
 cell_editing_started (GtkCellRenderer *cell,
                       GtkCellEditable *editable,
                       const gchar     *path,
-                      gpointer         data)
+                      gpointer         user_data)
 {
+	CEPageIP4 *self = CE_PAGE_IP4 (user_data);
+	CEPageIP4Private *priv = CE_PAGE_IP4_GET_PRIVATE (self);
+
 	if (!GTK_IS_ENTRY (editable)) {
 		g_warning ("%s: Unexpected cell editable type.", __func__);
 		return;
 	}
 
+	g_free (priv->last_edited);
+	priv->last_edited = NULL;
+
 	/* Set up the entry filter */
 	g_signal_connect (G_OBJECT (editable), "insert-text",
 	                  (GCallback) ip_address_filter_cb,
-	                  data);
+	                  user_data);
 }
 
 static void
@@ -580,7 +678,7 @@ routes_button_clicked_cb (GtkWidget *button, gpointer user_data)
 	char *tmp;
 
 	toplevel = gtk_widget_get_toplevel (CE_PAGE (self)->page);
-	g_return_if_fail (GTK_WIDGET_TOPLEVEL (toplevel));
+	g_return_if_fail (gtk_widget_is_toplevel (toplevel));
 
 	method = nm_setting_ip4_config_get_method (priv->setting);
 	if (!method || !strcmp (method, NM_SETTING_IP4_CONFIG_METHOD_AUTO))
@@ -632,7 +730,8 @@ finish_setup (CEPageIP4 *self, gpointer unused, GError *error, gpointer user_dat
 	g_object_set (renderer, "editable", TRUE, NULL);
 	g_signal_connect (renderer, "edited", G_CALLBACK (cell_edited), self);
 	g_object_set_data (G_OBJECT (renderer), "column", GUINT_TO_POINTER (COL_ADDRESS));
-	g_signal_connect (renderer, "editing-started", G_CALLBACK (cell_editing_started), store);
+	g_signal_connect (renderer, "editing-started", G_CALLBACK (cell_editing_started), self);
+	g_signal_connect (renderer, "editing-canceled", G_CALLBACK (cell_editing_canceled), self);
 	priv->addr_cells[COL_ADDRESS] = GTK_CELL_RENDERER (renderer);
 
 	offset = gtk_tree_view_insert_column_with_attributes (priv->addr_list,
@@ -648,7 +747,8 @@ finish_setup (CEPageIP4 *self, gpointer unused, GError *error, gpointer user_dat
 	g_object_set (renderer, "editable", TRUE, NULL);
 	g_signal_connect (renderer, "edited", G_CALLBACK (cell_edited), self);
 	g_object_set_data (G_OBJECT (renderer), "column", GUINT_TO_POINTER (COL_PREFIX));
-	g_signal_connect (renderer, "editing-started", G_CALLBACK (cell_editing_started), store);
+	g_signal_connect (renderer, "editing-started", G_CALLBACK (cell_editing_started), self);
+	g_signal_connect (renderer, "editing-canceled", G_CALLBACK (cell_editing_canceled), self);
 	priv->addr_cells[COL_PREFIX] = GTK_CELL_RENDERER (renderer);
 
 	offset = gtk_tree_view_insert_column_with_attributes (priv->addr_list,
@@ -664,7 +764,8 @@ finish_setup (CEPageIP4 *self, gpointer unused, GError *error, gpointer user_dat
 	g_object_set (renderer, "editable", TRUE, NULL);
 	g_signal_connect (renderer, "edited", G_CALLBACK (cell_edited), self);
 	g_object_set_data (G_OBJECT (renderer), "column", GUINT_TO_POINTER (COL_GATEWAY));
-	g_signal_connect (renderer, "editing-started", G_CALLBACK (cell_editing_started), store);
+	g_signal_connect (renderer, "editing-started", G_CALLBACK (cell_editing_started), self);
+	g_signal_connect (renderer, "editing-canceled", G_CALLBACK (cell_editing_canceled), self);
 	priv->addr_cells[COL_GATEWAY] = GTK_CELL_RENDERER (renderer);
 
 	offset = gtk_tree_view_insert_column_with_attributes (priv->addr_list,
@@ -691,11 +792,16 @@ finish_setup (CEPageIP4 *self, gpointer unused, GError *error, gpointer user_dat
 
 	g_signal_connect_swapped (priv->dhcp_client_id, "changed", G_CALLBACK (ce_page_changed), self);
 
+	g_signal_connect_swapped (priv->ip4_required, "toggled", G_CALLBACK (ce_page_changed), self);
+
 	g_signal_connect (priv->routes_button, "clicked", G_CALLBACK (routes_button_clicked_cb), self);
 }
 
 CEPage *
-ce_page_ip4_new (NMConnection *connection, GtkWindow *parent_window, GError **error)
+ce_page_ip4_new (NMConnection *connection,
+                 GtkWindow *parent_window,
+                 const char **out_secrets_setting_name,
+                 GError **error)
 {
 	CEPageIP4 *self;
 	CEPageIP4Private *priv;
@@ -741,10 +847,6 @@ ce_page_ip4_new (NMConnection *connection, GtkWindow *parent_window, GError **er
 	}
 
 	g_signal_connect (self, "initialized", G_CALLBACK (finish_setup), NULL);
-	if (!ce_page_initialize (parent, NULL, error)) {
-		g_object_unref (self);
-		return NULL;
-	}
 
 	return CE_PAGE (self);
 }
@@ -797,6 +899,7 @@ ui_to_setting (CEPageIP4 *self)
 	gboolean ignore_auto_dns = FALSE;
 	const char *dhcp_client_id = NULL;
 	char **items = NULL, **iter;
+	gboolean may_fail = FALSE;
 
 	/* Method */
 	if (gtk_combo_box_get_active_iter (priv->method, &tree_iter)) {
@@ -814,6 +917,9 @@ ui_to_setting (CEPageIP4 *self)
 	case IP4_METHOD_SHARED:
 		method = NM_SETTING_IP4_CONFIG_METHOD_SHARED;
 		break;
+	case IP4_METHOD_DISABLED:
+		method = NM_SETTING_IP4_CONFIG_METHOD_DISABLED;
+		break;
 	case IP4_METHOD_AUTO_ADDRESSES:
 		ignore_auto_dns = TRUE;
 		/* fall through */
@@ -932,6 +1038,8 @@ ui_to_setting (CEPageIP4 *self)
 			dhcp_client_id = NULL;
 	}
 
+	may_fail = !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->ip4_required));
+
 	/* Update setting */
 	g_object_set (priv->setting,
 				  NM_SETTING_IP4_CONFIG_METHOD, method,
@@ -940,6 +1048,7 @@ ui_to_setting (CEPageIP4 *self)
 				  NM_SETTING_IP4_CONFIG_DNS_SEARCH, search_domains,
 				  NM_SETTING_IP4_CONFIG_IGNORE_AUTO_DNS, ignore_auto_dns,
 				  NM_SETTING_IP4_CONFIG_DHCP_CLIENT_ID, dhcp_client_id,
+				  NM_SETTING_IP4_CONFIG_MAY_FAIL, may_fail,
 				  NULL);
 	valid = TRUE;
 
@@ -979,10 +1088,15 @@ dispose (GObject *object)
 {
 	CEPageIP4 *self = CE_PAGE_IP4 (object);
 	CEPageIP4Private *priv = CE_PAGE_IP4_GET_PRIVATE (self);
+	int i;
 
 	if (priv->window_group)
 		g_object_unref (priv->window_group);
 
+	/* Mark CEPageIP4 object as invalid; store this indication to cells to be usable in callbacks */
+	for (i = 0; i <= COL_LAST; i++)
+		g_object_set_data (G_OBJECT (priv->addr_cells[i]), "ce-page-not-valid", GUINT_TO_POINTER (1));
+
 	g_free (priv->connection_id);
 
 	G_OBJECT_CLASS (ce_page_ip4_parent_class)->dispose (object);
