@@ -30,7 +30,8 @@ typedef struct {
 } WirelessSecurityWPAEAP;
 
 WirelessSecurityWPAEAP * ws_wpa_eap_new (const char *glade_file,
-                                         NMConnection *connection);
+                                         NMConnection *connection,
+                                         gboolean is_editor);
 
 #endif /* WS_WPA_EAP_H */
 
